pub struct ShutdownSignals(/* private fields */);Expand description
Wrapper behind futures that upon completion will trigger binary shutdown.
Implementations
Sourceimpl ShutdownSignals
impl ShutdownSignals
Sourcepub async fn wait_for_signal(&mut self)
pub async fn wait_for_signal(&mut self)
Wait for any of the registered signals to be ready