pub type ConnectionCommandSender = UnboundedSender<ConnectionCommand>;Expand description
Used by the connection controller to report current state for client connections.
Aliased Type
pub struct ConnectionCommandSender(/* private fields */);pub type ConnectionCommandSender = UnboundedSender<ConnectionCommand>;Used by the connection controller to report current state for client connections.
pub struct ConnectionCommandSender(/* private fields */);