TaskManager

Struct TaskManager 

pub struct TaskManager { /* private fields */ }
👎Deprecated: use ShutdownManager instead
Listens to status and error messages from tasks, as well as notifying them to gracefully shutdown. Keeps track of if task stop unexpectedly, such as in a panic.

Implementations

Source

impl TaskManager

pub fn new(shutdown_timer_secs: u64) -> Self

pub fn named<S: Into<String>>(self, name: S) -> Self

pub async fn catch_interrupt( &mut self, ) -> Result<(), Box<dyn Error + Send + Sync>>

pub fn subscribe(&self) -> TaskClient

pub fn subscribe_named<S: Into<String>>(&self, suffix: S) -> TaskClient

pub fn signal_shutdown(&self) -> Result<(), SendError<()>>

pub async fn start_status_listener( &mut self, sender: StatusSender, start_status: TaskStatus, )

pub async fn wait_for_error(&mut self) -> Option<Box<dyn Error + Send + Sync>>

pub async fn wait_for_graceful_shutdown(&mut self)

pub async fn wait_for_shutdown(&mut self)